<?php
/**
 * Created by PhpStorm.
 * User: geovanesantos
 * Date: 27/01/2015
 * Time: 16:07
 */

namespace Gestao\Logic;


require_once __DIR__.'/../../../../projects/tecfood/public/bootstrap.php';

/**
 * Class IntegraMaoObraDRO
 *
 * Essa classe permite a adição programática de registros à tabela LANCDESPFATU
 *
 * @package Gestao\Logic
 */
class IntegraMaoObraDRO {

    /**
     * Insere um registro em LANCDESPFATU
     *
     * @param String $cdFilial
     *  Código da filial de ocorrência da despesa.
     * @param String $cdDespAdmFat
     *  Código da despesa previamente cadastrado no sistema Tecfood.
     * @param String $dtLancDesFat
     *  Data de ocorrência da despesa.
     * @param String $vrLancDesFat
     *  Valor da ocorrência da despesa.
     * @param String $vrPrevDesFat
     *  Valor previsto da despesa, campo opcional caso queira utilizar.
     * @return bool
     *  Retorna true se a inserção foi bem-sucedida.
     * @throws \SoapFault
     */
    function insereMaoDeObra($cdFilial, $cdDespAdmFat, $dtLancDesFat, $vrLancDesFat, $vrPrevDesFat)
    {
        //register_shutdown_function('checkError');

        $insertCommand = "insert into LANCDESPFATU(CDFILIAL, CDDESPADMFAT, DTLANCDESFAT, VRLANCDESFAT, VRPREVDESFAT, NRORG)
                      values(:P_CDFILIAL, :P_CDDESPADMFAT, :P_DTLANCDESFAT, :P_VRLANCDESFAT, :P_VRPREVDESFAT, :P_NRORG)";

        $arrayParams["P_CDFILIAL"] = $cdFilial;
        $arrayParams["P_CDDESPADMFAT"] = $cdDespAdmFat;
        $arrayParams["P_DTLANCDESFAT"] = $dtLancDesFat;

        //é necessario converter os valores de string para float
        //antes disso, é necessário trocar o separador de decimal de uma vírgula para um ponto, pro PHP ler corretamente o número
        $arrayParams["P_VRLANCDESFAT"] = (float)str_replace(',', '.', $vrLancDesFat);
        $arrayParams["P_VRPREVDESFAT"] = (float)str_replace(',', '.', $vrPrevDesFat);

        $arrayParams["P_NRORG"] = 1;

        $db = \Setforms2\Runtime\Connection::getInstance();

        try {
            $db->execute($insertCommand, $arrayParams);
        }

            //tenta primeiro pegar uma DBALException
            //se o programa tentar incluir um registro que já existe, essa exceção será disparada
            //portanto, é necessário checa-la para decidir se deve atualizar o registro
        catch (\Doctrine2\DBAL\DBALException $e) {
            try {
                $sqlMessage = $e->getPrevious()->getMessage();
                $sqlErrorCode = mb_substr($sqlMessage, 0, 9);

                //se não for uma exceção de banco com código ORA-00001, relança a exceção
                //se for, assume que foi um erro de repetição de PK e tenta atualizar
                if ($sqlErrorCode != "ORA-00001") throw $e;
                else {
                    $updateCommand = "update LANCDESPFATU set VRLANCDESFAT=:P_VRLANCDESFAT, VRPREVDESFAT=:P_VRPREVDESFAT
                  where CDFILIAL=:P_CDFILIAL AND CDDESPADMFAT=:P_CDDESPADMFAT AND DTLANCDESFAT=:P_DTLANCDESFAT AND NRORG=:P_NRORG";
                    $db->execute($updateCommand, $arrayParams);
                }
            }
                //só pra garantir, se algo der errado aqui dentro, pega a exception e termina o script com o erro
                //esse catch também pega a exceção rejogada, no caso de ter sido uma exceção de DB que não a ORA-00001
            catch (\Exception $e) {
                throw new \SoapFault("Sender", $e->getMessage());
            }
        } //se for outro erro, só retorna o erro
        catch (\Exception $e) {
            throw new \SoapFault("Sender", $e->getMessage());
        }
        //se tudo der certo, retorna OK
        return true;
    }
} 